@@ -28,15 +28,19 @@
 #include <config.h>
 #include "nautilus-throbber.h"
 
+#include <bonobo/bonobo-ui-toolbar-item.h>
+#include <eel/eel-debug.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-graphic-effects.h>
 #include <eel/eel-gtk-extensions.h>
+#include <eel/eel-accessibility.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtksignal.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
+#include <libnautilus/nautilus-view-standard-main.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
@@ -69,6 +73,7 @@ static void nautilus_throbber_load_images            (NautilusThrobber *throbber
 static void nautilus_throbber_unload_images          (NautilusThrobber *throbber);
 static void nautilus_throbber_theme_changed          (gpointer          user_data);
 static void nautilus_throbber_remove_update_callback (NautilusThrobber *throbber);
+static AtkObject *nautilus_throbber_get_accessible   (GtkWidget *widget);
 
 GNOME_CLASS_BOILERPLATE (NautilusThrobber, nautilus_throbber,
 			 GtkEventBox, GTK_TYPE_EVENT_BOX)
@@ -78,6 +83,7 @@ GNOME_CLASS_BOILERPLATE (NautilusThrobber, nautilus_throbber,
 /* routines to handle setting and getting the configuration properties of the Bonobo control */
 
 enum {
+	STYLE,
 	THROBBING,
 	LOCATION
 } MyArgs;
@@ -99,24 +105,24 @@ get_bonobo_properties (BonoboPropertyBag *bag,
 	NautilusThrobber *throbber = NAUTILUS_THROBBER (user_data);
 
 	switch (arg_id) {
-		case THROBBING:
-		{
-			BONOBO_ARG_SET_BOOLEAN (arg, throbber->details->timer_task != 0);
-			break;
-		}
+	case THROBBING:
+	{
+		BONOBO_ARG_SET_BOOLEAN (arg, throbber->details->timer_task != 0);
+		break;
+	}
 
-		case LOCATION:
-		{
-			char *location = nautilus_theme_get_theme_data ("throbber", "url");
-			if (location != NULL) {
-				BONOBO_ARG_SET_STRING (arg, location);
-				g_free (location);
-			} else {
-				BONOBO_ARG_SET_STRING (arg, "");			
-			}
-		
+	case LOCATION:
+	{
+		char *location = nautilus_theme_get_theme_data ("throbber", "url");
+		if (location != NULL) {
+			BONOBO_ARG_SET_STRING (arg, location);
+			g_free (location);
+		} else {
+			BONOBO_ARG_SET_STRING (arg, "");			
 		}
 		
+	}
+
 		default:
 			g_warning ("Unhandled arg %d", arg_id);
 			break;
@@ -132,23 +138,28 @@ set_bonobo_properties (BonoboPropertyBag *bag,
 {
 	NautilusThrobber *throbber = NAUTILUS_THROBBER (user_data);
 	switch (arg_id) {
-		case THROBBING:
-		{
-			gboolean throbbing;
-
-			throbbing = BONOBO_ARG_GET_BOOLEAN (arg);
-			
-			if (throbbing != is_throbbing (throbber)) {
+	case THROBBING:
+	{
+		gboolean throbbing;
+		
+		throbbing = BONOBO_ARG_GET_BOOLEAN (arg);
+		
+		if (throbbing != is_throbbing (throbber)) {
 				if (throbbing) {
 					nautilus_throbber_start (throbber);
 				} else {
 					nautilus_throbber_stop (throbber);
 				}
-			}
-						
-			break;
 		}
-
+		
+		break;
+	}
+	case STYLE:
+	{
+		nautilus_throbber_set_small_mode (throbber, BONOBO_ARG_GET_INT (arg) !=
+						  BONOBO_UI_TOOLBAR_ITEM_STYLE_ICON_AND_TEXT_VERTICAL);
+		break;
+	}
 		default:
 			g_warning ("Unhandled arg %d", arg_id);
 			break;
@@ -237,23 +248,17 @@ nautilus_throbber_instance_init (NautilusThrobber *throbber)
 				 "Throbber active", 0);
 	bonobo_property_bag_add (throbber->details->property_bag, "location", LOCATION, BONOBO_ARG_STRING, NULL,
 				 "associated URL", 0);
-	
+	bonobo_property_bag_add (throbber->details->property_bag, "style", STYLE, BONOBO_ARG_INT, NULL, NULL,
+				 Bonobo_PROPERTY_WRITEABLE);
 	nautilus_throbber_load_images (throbber);
 	gtk_widget_show (widget);
-	
+
 	/* add a callback for when the theme changes */
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_THEME,
 				      nautilus_throbber_theme_changed,
 				      throbber);
 }
 
-/* allocate a new throbber */
-GtkWidget *
-nautilus_throbber_new (void)
-{
-	return gtk_widget_new (nautilus_throbber_get_type (), NULL);
-}
-
 /* handler for handling theme changes */
 static void
 nautilus_throbber_theme_changed (gpointer user_data)
@@ -468,8 +473,8 @@ load_themed_image (const char *file_name, const char *image_theme, gboolean smal
 		
 		if (small_mode && pixbuf) {
 			temp_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
-							       gdk_pixbuf_get_width (pixbuf) / 2,
-							       gdk_pixbuf_get_height (pixbuf) / 2,
+							       gdk_pixbuf_get_width (pixbuf) * 2 / 3,
+							       gdk_pixbuf_get_height (pixbuf) * 2 / 3,
 							       GDK_INTERP_BILINEAR);
 			g_object_unref (pixbuf);
 			pixbuf = temp_pixbuf;
@@ -517,6 +522,7 @@ nautilus_throbber_load_images (NautilusThrobber *throbber)
 	for (index = 1; index <= throbber->details->max_frame; index++) {
 		throbber_frame_name = make_throbber_frame_name (index);
 		pixbuf = load_themed_image (throbber_frame_name, image_theme, throbber->details->small_mode);
+
 		g_free (throbber_frame_name);
 		if (pixbuf == NULL) {
 			throbber->details->max_frame = index - 1;
@@ -581,28 +587,35 @@ nautilus_throbber_button_press_event (GtkWidget *widget, GdkEventButton *event)
 		(GTK_WIDGET_CLASS, button_press_event, (widget, event), FALSE);
 }
 
+static void
+nautilus_throbber_set_location (NautilusThrobber *throbber)
+{
+	char *location;
+	BonoboArg *location_arg;
+
+	location = nautilus_theme_get_theme_data ("throbber", "url");
+	if (location != NULL) {
+		location_arg = bonobo_arg_new (BONOBO_ARG_STRING);
+		BONOBO_ARG_SET_STRING (location_arg, location);
+		bonobo_event_source_notify_listeners_full (
+			throbber->details->property_bag->es,
+			"Bonobo/Property", "change", "location",
+			location_arg, NULL);
+		bonobo_arg_release (location_arg);
+		g_free (location);
+	}
+}
+
 static gboolean
 nautilus_throbber_button_release_event (GtkWidget *widget, GdkEventButton *event)
 {	
 	NautilusThrobber *throbber;
-	char *location;
-	BonoboArg *location_arg;
 	
 	throbber = NAUTILUS_THROBBER (widget);
 
 	if (event->button == 1) {
 		if (throbber->details->button_in) {
-			location = nautilus_theme_get_theme_data ("throbber", "url");
-			if (location != NULL) {
-				location_arg = bonobo_arg_new (BONOBO_ARG_STRING);
-				BONOBO_ARG_SET_STRING (location_arg, location);
-				bonobo_event_source_notify_listeners_full (
-					throbber->details->property_bag->es,
-					"Bonobo/Property", "change", "location",
-					location_arg, NULL);
-				bonobo_arg_release (location_arg);
-				g_free (location);
-			}
+			nautilus_throbber_set_location (throbber);
 		}
 		throbber->details->button_down = FALSE;
 		gtk_widget_queue_draw (widget);
@@ -677,4 +690,184 @@ nautilus_throbber_class_init (NautilusThrobberClass *class)
 	widget_class->leave_notify_event = nautilus_throbber_leave_notify_event;
 	widget_class->size_request = nautilus_throbber_size_request;	
 	widget_class->map = nautilus_throbber_map;
+	widget_class->get_accessible = nautilus_throbber_get_accessible;
+}
+
+static AtkObjectClass *a11y_parent_class = NULL;
+
+static void
+nautilus_throbber_accessible_initialize (AtkObject *accessible,
+					 gpointer   widget)
+{
+	atk_object_set_name (accessible, _("throbber"));
+	atk_object_set_description (accessible, _("provides visual status"));
+
+	a11y_parent_class->initialize (accessible, widget);
+}
+
+static void
+nautilus_throbber_accessible_class_init (AtkObjectClass *klass)
+{
+	a11y_parent_class = g_type_class_peek_parent (klass);
+
+	klass->initialize = nautilus_throbber_accessible_initialize;
+}
+
+static void
+nautilus_throbber_accessible_image_get_size (AtkImage *image,
+					     gint     *width,
+					     gint     *height)
+{
+	GtkWidget *widget;
+
+	widget = GTK_ACCESSIBLE (image)->widget;
+	if (!widget) {
+		*width = *height = 0;
+	} else {
+		*width = widget->allocation.width;
+		*height = widget->allocation.height;
+	}
+}
+
+static void
+nautilus_throbber_accessible_image_interface_init (AtkImageIface *iface)
+{
+	iface->get_image_size = nautilus_throbber_accessible_image_get_size;
+}
+
+/* AtkAction interface */
+
+enum {
+	ACTION_ACTIVATE,
+	LAST_ACTION
+};
+
+static const char *nautilus_throbber_accessible_action_names[] = {
+	"activate",
+	NULL
+};
+
+static const char *nautilus_throbber_accessible_action_descriptions[] = {
+	"Activate selected items",
+	NULL
+};
+
+
+static gboolean
+nautilus_throbber_accessible_do_action (AtkAction *accessible, int i)
+{
+	GtkWidget *widget;
+
+	g_return_val_if_fail (i < LAST_ACTION, FALSE);
+
+	widget = GTK_ACCESSIBLE (accessible)->widget;
+	if (!widget) {
+		return FALSE;
+	}
+	
+	switch (i) {
+	case ACTION_ACTIVATE :
+		nautilus_throbber_set_location (NAUTILUS_THROBBER (widget));
+		return TRUE;
+	default:
+		return FALSE;
+	}
+}
+
+static int
+nautilus_throbber_accessible_get_n_actions (AtkAction *accessible)
+{
+	return LAST_ACTION;
+}
+
+static const char *
+nautilus_throbber_accessible_action_get_description (AtkAction *accessible, 
+							   int i)
+{
+	g_return_val_if_fail (i < LAST_ACTION, NULL);
+
+	return nautilus_throbber_accessible_action_descriptions[i];
+}
+
+static const char *
+nautilus_throbber_accessible_action_get_name (AtkAction *accessible, int i)
+{
+	g_return_val_if_fail (i < LAST_ACTION, NULL);
+
+	return nautilus_throbber_accessible_action_names [i];
+}
+
+static const char *
+nautilus_throbber_accessible_action_get_keybinding (AtkAction *accessible, 
+						    int        i)
+{
+	return NULL;
+}
+
+static gboolean
+nautilus_throbber_accessible_action_set_description (AtkAction  *accessible, 
+						     int         i, 
+						     const char *description)
+{
+	return FALSE;
+}
+
+static void
+nautilus_throbber_accessible_action_interface_init (AtkActionIface *iface)
+{
+	iface->do_action = nautilus_throbber_accessible_do_action;
+	iface->get_n_actions = nautilus_throbber_accessible_get_n_actions;
+	iface->get_description = nautilus_throbber_accessible_action_get_description;
+	iface->get_name = nautilus_throbber_accessible_action_get_name;
+	iface->get_keybinding = nautilus_throbber_accessible_action_get_keybinding;
+	iface->set_description = nautilus_throbber_accessible_action_set_description;
+}
+
+static GType
+nautilus_throbber_accessible_get_type (void)
+{
+        static GType type = 0;
+
+	/* Action interface
+	   Name etc. ... */
+        if (!type) {
+                static GInterfaceInfo atk_action_info = {
+                        (GInterfaceInitFunc) nautilus_throbber_accessible_action_interface_init,
+                        (GInterfaceFinalizeFunc) NULL,
+                        NULL
+                };              
+
+		static const GInterfaceInfo atk_image_info = {
+			(GInterfaceInitFunc) nautilus_throbber_accessible_image_interface_init,
+			(GInterfaceFinalizeFunc) NULL,
+			NULL
+		};
+
+		type = eel_accessibility_create_derived_type 
+			("NautilusThrobberAccessible",
+			 GTK_TYPE_IMAGE,
+			 nautilus_throbber_accessible_class_init);
+		
+                g_type_add_interface_static (type, ATK_TYPE_ACTION,
+                                             &atk_action_info);
+                g_type_add_interface_static (type, ATK_TYPE_IMAGE,
+                                             &atk_image_info);
+        }
+
+        return type;
+}
+
+static AtkObject *
+nautilus_throbber_get_accessible (GtkWidget *widget)
+{
+	AtkObject *accessible;
+	
+	if ((accessible = eel_accessibility_get_atk_object (widget))) {
+		return accessible;
+	}
+	
+	accessible = g_object_new 
+		(nautilus_throbber_accessible_get_type (), NULL);
+	
+	return eel_accessibility_set_atk_object_return (widget, accessible);
 }
