@@ -45,8 +45,8 @@
 #define PREFERENCES_SORT_ORDER_MANUALLY 100
 
 /* Path for gnome-vfs preferences */
-static const char *EXTRA_MONITOR_PATHS[] = { "/system/gnome-vfs",
-					     "/desktop/gnome/file-views",
+static const char *EXTRA_MONITOR_PATHS[] = { "/system/gnome_vfs",
+					     "/desktop/gnome/file_views",
 					     NULL };
 
 /* Forward declarations */
@@ -97,9 +97,9 @@ static PreferenceImportType OLD_PREFERENCES_TO_IMPORT[] = {
 
 /* Enumerations used to qualify some INTEGER preferences */
 static EelEnumerationEntry speed_tradeoff_enum_entries[] = {
-	{ "always",	    N_("Always"),		NAUTILUS_SPEED_TRADEOFF_ALWAYS },
-	{ "local_only",	    N_("Local Files Only"),	NAUTILUS_SPEED_TRADEOFF_LOCAL_ONLY },
-	{ "never",	    N_("Never"),		NAUTILUS_SPEED_TRADEOFF_NEVER },
+	{ "always",	    N_("_Always"),		NAUTILUS_SPEED_TRADEOFF_ALWAYS },
+	{ "local_only",	    N_("_Local Files Only"),	NAUTILUS_SPEED_TRADEOFF_LOCAL_ONLY },
+	{ "never",	    N_("_Never"),		NAUTILUS_SPEED_TRADEOFF_NEVER },
 	{ NULL }
 };
 
@@ -134,11 +134,11 @@ static EelEnumerationEntry file_size_enum_entries[] = {
 
 static EelEnumerationEntry click_policy_enum_entries[] = {
 	{ "single",
-	  N_("Activate items with a single click"),
+	  N_("Activate items with a _single click"),
 	  NAUTILUS_CLICK_POLICY_SINGLE
 	},
 	{ "double",
-	  N_("Activate items with a double click"),
+	  N_("Activate items with a _double click"),
 	  NAUTILUS_CLICK_POLICY_DOUBLE
 	},
 	{ NULL }
@@ -146,15 +146,15 @@ static EelEnumerationEntry click_policy_enum_entries[] = {
 
 static EelEnumerationEntry executable_text_activation_enum_entries[] = {
 	{ "launch",
-	  N_("Execute files when they are clicked"),
+	  N_("E_xecute files when they are clicked"),
 	  NAUTILUS_EXECUTABLE_TEXT_LAUNCH
 	},
 	{ "display",
-	  N_("Display files when they are clicked"),
+	  N_("Display _files when they are clicked"),
 	  NAUTILUS_EXECUTABLE_TEXT_DISPLAY
 	},
 	{ "ask",
-	  N_("Ask each time"),
+	  N_("_Ask each time"),
 	  NAUTILUS_EXECUTABLE_TEXT_ASK
 	},
 	{ NULL }
@@ -369,17 +369,11 @@ static const PreferenceDefault preference_defaults[] = {
 	  "speed_tradeoff"
 	},
 	{ NAUTILUS_PREFERENCES_IMAGE_FILE_THUMBNAIL_LIMIT,
-	  PREFERENCE_STRING,
-	  "3145728",
+	  PREFERENCE_INTEGER,
+	  GINT_TO_POINTER(3145728),
 	  NULL, NULL,
 	  "file_size"
 	},
-	{ NAUTILUS_PREFERENCES_USE_PUBLIC_METADATA,
-	  PREFERENCE_STRING,
-	  "local_only",
-	  NULL, NULL,
-	  "speed_tradeoff"
-	},
 	{ NAUTILUS_PREFERENCES_PREVIEW_SOUND,
 	  PREFERENCE_STRING,
 	  "local_only",
@@ -402,10 +396,6 @@ static const PreferenceDefault preference_defaults[] = {
 	  PREFERENCE_BOOLEAN,
 	  GINT_TO_POINTER (FALSE)
 	},
-	{ NAUTILUS_PREFERENCES_CAN_ADD_CONTENT,
-	  PREFERENCE_BOOLEAN,
-	  GINT_TO_POINTER (FALSE)
-	},
 	{ NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
 	  PREFERENCE_STRING,
 	  "search_by_text",
@@ -452,50 +442,17 @@ static const PreferenceDefault preference_defaults[] = {
 	  GINT_TO_POINTER (FALSE)
 	},
 
-	/* Proxy defaults */
-	{ NAUTILUS_PREFERENCES_HTTP_USE_PROXY,
-	  PREFERENCE_BOOLEAN,
-	  GINT_TO_POINTER (FALSE)
-	},
-	{ NAUTILUS_PREFERENCES_HTTP_PROXY_PORT,
-	  PREFERENCE_INTEGER,
-	  GINT_TO_POINTER (8080)
-	},
-	{ NAUTILUS_PREFERENCES_HTTP_PROXY_USE_AUTH,
-	  PREFERENCE_BOOLEAN,
-	  GINT_TO_POINTER (FALSE)
-	},
-
 	/* Home URI */
 	{ NAUTILUS_PREFERENCES_HOME_URI,
 	  PREFERENCE_STRING,
 	  NULL, default_home_location_callback, g_free
 	},
-
-	/* Default fonts */
-	{ NAUTILUS_PREFERENCES_DEFAULT_FONT,
+	{ NAUTILUS_PREFERENCES_DESKTOP_FONT,
 	  PREFERENCE_STRING,
 	  NULL, default_font_callback, g_free
 	},
-	{ NAUTILUS_PREFERENCES_DEFAULT_FONT_SIZE,
-	  PREFERENCE_STRING,
-	  "12",
-	  NULL, NULL,
-	  "standard_font_size"
-	},
-	
+
 	/* Icon View Default Preferences */
-
-	{ NAUTILUS_PREFERENCES_ICON_VIEW_FONT,
-	  PREFERENCE_STRING,
-	  NULL, default_font_callback, g_free
-	},
-	{ NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL_FONT_SIZE,
-	  PREFERENCE_STRING,
-	  "12",
-	  NULL, NULL,
-	  "standard_font_size"
-	},
 	{ NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_ORDER,
 	  PREFERENCE_STRING,
 	  "name",
@@ -528,16 +485,6 @@ static const PreferenceDefault preference_defaults[] = {
 	},
 
 	/* List View Default Preferences */
-	{ NAUTILUS_PREFERENCES_LIST_VIEW_FONT,
-	  PREFERENCE_STRING,
-	  NULL, default_font_callback, g_free
-	},
-	{ NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL_FONT_SIZE,
-	  PREFERENCE_STRING,
-	  "12",
-	  NULL, NULL,
-	  "standard_font_size"
-	},
 	{ NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_ORDER,
 	  PREFERENCE_STRING,
 	  "name",
@@ -550,7 +497,7 @@ static const PreferenceDefault preference_defaults[] = {
 	},
 	{ NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
 	  PREFERENCE_STRING,
-	  "smaller",
+	  "small",
 	  NULL, NULL,
 	  "default_zoom_level"
 	},
@@ -613,7 +560,8 @@ global_preferences_register_enumerations (void)
 	for (i = 0; preference_defaults[i].name != NULL; i++) {
 		if (eel_strlen (preference_defaults[i].enumeration_id) > 0) {
 			g_assert (preference_defaults[i].type == PREFERENCE_STRING
-				  || preference_defaults[i].type == PREFERENCE_STRING_LIST);
+				  || preference_defaults[i].type == PREFERENCE_STRING_LIST
+				  || preference_defaults[i].type == PREFERENCE_INTEGER);
 			eel_preferences_set_enumeration_id (preference_defaults[i].name,
 							    preference_defaults[i].enumeration_id);
 		}
@@ -702,7 +650,7 @@ global_preferences_install_defaults (void)
 static gpointer
 default_font_callback (void)
 {
-	return g_strdup ("sans");
+	return g_strdup ("sans 12");
 }
 
 static int
