@@ -81,13 +81,13 @@
 /* Paths to use when creating & referring to Bonobo menu items */
 #define MENU_PATH_STRETCH_ICON 			"/menu/Edit/Edit Items Placeholder/Stretch"
 #define MENU_PATH_UNSTRETCH_ICONS 		"/menu/Edit/Edit Items Placeholder/Unstretch"
-#define MENU_PATH_LAY_OUT			"/menu/View/View Items Placeholder/Lay Out"
-#define MENU_PATH_MANUAL_LAYOUT 		"/menu/View/View Items Placeholder/Lay Out/Manual Layout"
-#define MENU_PATH_TIGHTER_LAYOUT 		"/menu/View/View Items Placeholder/Lay Out/Tighter Layout"
-#define MENU_PATH_SORT_REVERSED			"/menu/View/View Items Placeholder/Lay Out/Reversed Order"
+#define MENU_PATH_LAY_OUT			"/menu/View/View Items Placeholder/Arrange Items"
+#define MENU_PATH_MANUAL_LAYOUT 		"/menu/View/View Items Placeholder/Arrange Items/Manual Layout"
+#define MENU_PATH_TIGHTER_LAYOUT 		"/menu/View/View Items Placeholder/Arrange Items/Tighter Layout"
+#define MENU_PATH_SORT_REVERSED			"/menu/View/View Items Placeholder/Arrange Items/Reversed Order"
 #define MENU_PATH_CLEAN_UP			"/menu/View/View Items Placeholder/Clean Up"
 
-#define POPUP_PATH_LAY_OUT			"/popups/background/Before Zoom Items/View Items/Lay Out"
+#define POPUP_PATH_LAY_OUT			"/popups/background/Before Zoom Items/View Items/Arrange Items"
 
 #define COMMAND_PREFIX                          "/commands/"
 #define COMMAND_STRETCH_ICON 			"/commands/Stretch"
@@ -187,8 +187,6 @@ static void                 fm_icon_view_set_directory_sort_by                 (
 static void                 fm_icon_view_set_zoom_level                        (FMIconView           *view,
 										NautilusZoomLevel     new_level,
 										gboolean              always_set_level);
-static void                 fm_icon_view_update_icon_container_fonts           (FMIconView           *icon_view);
-static void                 fm_icon_view_update_icon_container_font_size_table (FMIconView           *icon_view);
 static void                 fm_icon_view_update_click_mode                     (FMIconView           *icon_view);
 static void                 fm_icon_view_set_directory_tighter_layout          (FMIconView           *icon_view,
 										NautilusFile         *file,
@@ -240,10 +238,18 @@ fm_icon_view_finalize (GObject *object)
 static NautilusIconContainer *
 get_icon_container (FMIconView *icon_view)
 {
-	g_return_val_if_fail (FM_IS_ICON_VIEW (icon_view), NULL);
-	g_return_val_if_fail (NAUTILUS_IS_ICON_CONTAINER (GTK_BIN (icon_view)->child), NULL);
+	GtkBin *bin;
 
-	return NAUTILUS_ICON_CONTAINER (GTK_BIN (icon_view)->child);
+	g_return_val_if_fail (FM_IS_ICON_VIEW (icon_view), NULL);
+
+	bin = GTK_BIN (icon_view);
+	if (bin->child) {
+		g_return_val_if_fail (NAUTILUS_IS_ICON_CONTAINER (GTK_BIN (icon_view)->child), NULL);
+		
+		return NAUTILUS_ICON_CONTAINER (GTK_BIN (icon_view)->child);
+	} else {
+		return NULL;
+	}
 }
 
 static gboolean
@@ -449,6 +455,8 @@ fm_icon_view_clear (FMDirectoryView *view)
 	g_return_if_fail (FM_IS_ICON_VIEW (view));
 
 	icon_container = get_icon_container (FM_ICON_VIEW (view));
+	if (!icon_container)
+		return;
 
 	/* Clear away the existing icons. */
 	nautilus_icon_container_for_each (icon_container, unref_cover, NULL);
@@ -951,6 +959,95 @@ fm_icon_view_end_loading (FMDirectoryView *view)
 	icon_view->details->loading = FALSE;
 }
 
+static void
+fm_icon_view_update_font_size_table (FMIconView *view)
+{
+	NautilusIconContainer *container;
+	int font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST + 1];
+
+	container = get_icon_container (view);
+	g_assert (container != NULL);
+
+	switch (get_default_zoom_level ())
+	{
+	case NAUTILUS_ZOOM_LEVEL_LARGEST:
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLEST] = -5 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLER]  = -4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALL]    = -4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_STANDARD] = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGE]    = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGER]   = -2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST]  =  0 * PANGO_SCALE;
+		break;
+	case NAUTILUS_ZOOM_LEVEL_LARGER:
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLEST] = -4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLER]  = -4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALL]    = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_STANDARD] = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGE]    = -2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGER]   =  0 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST]  =  2 * PANGO_SCALE;
+		break;
+	case NAUTILUS_ZOOM_LEVEL_LARGE:
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLEST] = -4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLER]  = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALL]    = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_STANDARD] = -2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGE]    =  0 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGER]   =  2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST]  =  4 * PANGO_SCALE;
+		break;
+	case NAUTILUS_ZOOM_LEVEL_STANDARD:
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLEST] = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLER]  = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALL]    = -2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_STANDARD] =  0 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGE]    =  2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGER]   =  4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST]  =  4 * PANGO_SCALE;
+		break;
+	case NAUTILUS_ZOOM_LEVEL_SMALL:
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLEST] = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLER]  = -2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALL]    =  0 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_STANDARD] =  2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGE]    =  4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGER]   =  4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST]  =  5 * PANGO_SCALE;
+		break;
+	case NAUTILUS_ZOOM_LEVEL_SMALLER:
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLEST] = -2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLER]  =  0 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALL]    =  2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_STANDARD] =  4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGE]    =  4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGER]   =  5 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST]  =  5 * PANGO_SCALE;
+		break;
+	case NAUTILUS_ZOOM_LEVEL_SMALLEST:
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLEST] =  0 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLER]  =  2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALL]    =  4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_STANDARD] =  4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGE]    =  5 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGER]   =  5 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST]  =  6 * PANGO_SCALE;
+		break;
+	default:
+		g_warning ("invalid default list-view zoom level");
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLEST] = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALLER]  = -3 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_SMALL]    = -2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_STANDARD] =  0 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGE]    =  2 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGER]   =  4 * PANGO_SCALE;
+		font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST]  =  4 * PANGO_SCALE;
+		break;
+	}
+
+	nautilus_icon_container_set_font_size_table (container, font_size_table);
+}
+
 static NautilusZoomLevel
 fm_icon_view_get_zoom_level (FMIconView *view)
 {
@@ -1284,8 +1381,8 @@ fm_icon_view_update_menus (FMDirectoryView *view)
 		 MENU_PATH_UNSTRETCH_ICONS,
 		 COMMAND_UNSTRETCH_ICONS,
 		 eel_g_list_more_than_one_item (selection)
-		 	? _("_Restore Icons' Original Sizes")
-		 	: _("_Restore Icon's Original Size"));
+		 	? _("Restore Icons' _Original Sizes")
+		 	: _("Restore Icon's _Original Size"));
 	nautilus_bonobo_set_sensitive (icon_view->details->ui, 
 				       COMMAND_UNSTRETCH_ICONS,
 				       icon_container != NULL
@@ -1837,7 +1934,7 @@ get_icon_drop_target_uri_callback (NautilusIconContainer *container,
 		       		   NautilusFile *file,
 		       		   FMIconView *icon_view)
 {
-	char *uri, *path, *target_uri;
+	char *uri, *target_uri;
 	
 	g_assert (NAUTILUS_IS_ICON_CONTAINER (container));
 	g_assert (NAUTILUS_IS_FILE (file));
@@ -1848,14 +1945,12 @@ get_icon_drop_target_uri_callback (NautilusIconContainer *container,
 	/* Check for Nautilus link */
 	if (nautilus_file_is_nautilus_link (file)) {
 		/* FIXME bugzilla.gnome.org 43020: This does sync. I/O and works only locally. */
-		path = gnome_vfs_get_local_path_from_uri (uri);
-		if (path != NULL) {
-			target_uri = nautilus_link_local_get_link_uri (path);
+		if (nautilus_file_is_local (file)) {
+			target_uri = nautilus_link_local_get_link_uri (uri);
 			if (target_uri != NULL) {
 				g_free (uri);
 				uri = target_uri;
 			}
-			g_free (path);
 		}
 	}
 
@@ -1887,22 +1982,6 @@ fm_icon_view_image_display_policy_changed (FMDirectoryView *directory_view)
 	nautilus_icon_container_request_update_all (get_icon_container (FM_ICON_VIEW (directory_view)));
 }
 
-static void
-font_changed_callback (gpointer callback_data)
-{
- 	g_return_if_fail (FM_IS_ICON_VIEW (callback_data));
-
-	fm_icon_view_update_icon_container_fonts (FM_ICON_VIEW (callback_data));
-}
-
-static void
-default_zoom_level_font_size_changed_callback (gpointer callback_data)
-{
- 	g_return_if_fail (FM_IS_ICON_VIEW (callback_data));
-
-	fm_icon_view_update_icon_container_font_size_table (FM_ICON_VIEW (callback_data));
-}
-
 static void
 fm_icon_view_click_policy_changed (FMDirectoryView *directory_view)
 {
@@ -2022,10 +2101,9 @@ default_zoom_level_changed_callback (gpointer callback_data)
 		level = nautilus_file_get_integer_metadata (file, 
 							    NAUTILUS_METADATA_KEY_ICON_VIEW_ZOOM_LEVEL, 
 							    get_default_zoom_level ());
+		fm_icon_view_update_font_size_table (icon_view);
 		fm_icon_view_set_zoom_level (icon_view, level, TRUE);
 	}
-
-	fm_icon_view_update_icon_container_font_size_table (icon_view);
 }
 
 static void
@@ -2072,88 +2150,6 @@ icon_view_move_copy_items (NautilusIconContainer *container,
 		copy_action, x, y, view);
 }
 
-static void
-fm_icon_view_update_icon_container_fonts (FMIconView *icon_view)
-{
-	NautilusIconContainer *icon_container;
-	char *font_name;
-	
-	icon_container = get_icon_container (icon_view);
-	g_assert (icon_container != NULL);
-
-	font_name = eel_preferences_get (NAUTILUS_PREFERENCES_ICON_VIEW_FONT);
-
-	nautilus_icon_container_set_font_name (icon_container, font_name);
-
-	g_free (font_name);
-}
-
-static int default_zoom_level_font_size = 12;
-
-static int
-get_default_zoom_level_font_size (void)
-{
-	static gboolean auto_storaged_added = FALSE;
-
-	if (auto_storaged_added == FALSE) {
-		auto_storaged_added = TRUE;
-		eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL_FONT_SIZE,
-					       &default_zoom_level_font_size);
-	}
-
-	return default_zoom_level_font_size;
-}
-
-static void
-fm_icon_view_update_icon_container_font_size_table (FMIconView *icon_view)
-{
-	/* Deltas in points from the default zoom level.  The actual font size table
-	 * is computed by taking the default_zoom level and assigning the default
-	 * zoom level font size for that.  Then all values above the default zoom
-	 * level are computed by using the positive deltas.  The values below the 
-	 * default zoom level as respectively computed by using the negative deltas
-	 */
-	static int positive_font_size_delta_table[NAUTILUS_ZOOM_LEVEL_LARGEST + 1] = {
-		0, +2, +4, +4, +6, +6, +8 };
-	static int negative_font_size_delta_table[NAUTILUS_ZOOM_LEVEL_LARGEST + 1] = {
-		0, -2, -3, -3, -4, -4, -5 };
-
-	const int min_font_size = 5;
-	const int max_font_size = 30;
-
-	int font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST + 1];
-	NautilusIconContainer *icon_container;
-	int default_zoom_level_font_size;
-	int i;
- 	int j;
-	NautilusZoomLevel default_level;
-
-	icon_container = get_icon_container (icon_view);
-	g_assert (icon_container != NULL);
-
-	default_zoom_level_font_size = get_default_zoom_level_font_size ();
-	default_level = get_default_zoom_level ();
-
-	g_return_if_fail (default_level >= NAUTILUS_ZOOM_LEVEL_SMALLEST);
-	g_return_if_fail (default_level <= NAUTILUS_ZOOM_LEVEL_LARGEST);
-
-	for (i = default_level, j = 0; i <= NAUTILUS_ZOOM_LEVEL_LARGEST; i++,j++) {
-		font_size_table[i] = CLAMP (default_zoom_level_font_size + positive_font_size_delta_table[j],
-					    min_font_size,
-					    max_font_size);
-	}
-
-	for (i = default_level - 1, j = 0; i >= NAUTILUS_ZOOM_LEVEL_SMALLEST; i--,j++) {
-		font_size_table[i] = CLAMP (default_zoom_level_font_size + negative_font_size_delta_table[j],
-					    min_font_size,
-					    max_font_size);
-
-	}
-	
-	nautilus_icon_container_set_font_size_table (icon_container, font_size_table);
-	nautilus_icon_container_request_update_all (icon_container);
-}
-
 static void
 fm_icon_view_update_click_mode (FMIconView *icon_view)
 {
@@ -2223,9 +2219,8 @@ create_icon_container (FMIconView *icon_view)
 	gtk_container_add (GTK_CONTAINER (icon_view),
 			   GTK_WIDGET (icon_container));
 
-	fm_icon_view_update_icon_container_fonts (icon_view);
-	fm_icon_view_update_icon_container_font_size_table (icon_view);
 	fm_icon_view_update_click_mode (icon_view);
+	fm_icon_view_update_font_size_table (icon_view);
 
 	gtk_widget_show (GTK_WIDGET (icon_container));
 }
@@ -2241,9 +2236,9 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 	GnomeDesktopItem *entry;
 	GdkPoint point;
 	char *uri;
+	char *path;
 	char *stripped_uri;
 	char *container_uri_string;
-	char *container_path;
 	const char *last_slash, *link_name;
 	int n_uris;
 	gboolean all_local;
@@ -2280,8 +2275,8 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 		eel_show_warning_dialog (_("An invalid drag type was used."),
 					 _("Drag and Drop error"),
 					 fm_directory_view_get_containing_window (FM_DIRECTORY_VIEW (view)));
-		g_free (container_uri);
 		gnome_vfs_uri_unref (container_uri);
+		g_free (container_uri_string);
 		return;
 	}
 
@@ -2329,14 +2324,23 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 		if (points)
 			g_array_free (points, TRUE);
 	} else {
-		container_path = gnome_vfs_get_local_path_from_uri (container_uri_string);
 		for (node = real_uri_list; node != NULL; node = node->next) {
 			/* Make a link using the desktop file contents? */
 			uri = node->data;
-			entry = gnome_desktop_item_new_from_uri (uri, 0, NULL);
+			path = gnome_vfs_get_local_path_from_uri (uri);
+			
+			if (path != NULL) {
+				entry = gnome_desktop_item_new_from_file (path,
+									  GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS,
+									  NULL);
+				g_free (path);
+			} else {
+				entry = NULL;
+			}
+
 			if (entry != NULL) {
 				/* FIXME: Handle name conflicts? */
-				nautilus_link_local_create_from_gnome_entry (entry, container_path, &point);
+				nautilus_link_local_create_from_gnome_entry (entry, container_uri_string, &point);
 
 				gnome_desktop_item_unref (entry);
 				continue;
@@ -2354,15 +2358,14 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 			
 			if (!eel_str_is_empty (link_name)) {
 				/* FIXME: Handle name conflicts? */
-				nautilus_link_local_create (container_path, link_name,
-							    "gnome-http-url", uri,
+				nautilus_link_local_create (container_uri_string, link_name,
+							    NULL, uri,
 							    &point, NAUTILUS_LINK_GENERIC);
 			}
 			
 			g_free (stripped_uri);
 			break;
 		}
-		g_free (container_path);
 	}
 	
 	nautilus_icon_dnd_uri_list_free_strings (real_uri_list);
@@ -2447,12 +2450,6 @@ fm_icon_view_instance_init (FMIconView *icon_view)
 		setup_sound_preview = TRUE;
 	}
 
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_FONT,
-						  font_changed_callback, 
-						  icon_view, G_OBJECT (icon_view));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL_FONT_SIZE,
-						  default_zoom_level_font_size_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_ORDER,
 						  default_sort_order_changed_callback,
 						  icon_view, G_OBJECT (icon_view));
