@@ -94,7 +94,13 @@ fm_list_model_get_column_type (GtkTreeModel *tree_model, int index)
 	case FM_LIST_MODEL_TYPE_COLUMN:
 	case FM_LIST_MODEL_DATE_MODIFIED_COLUMN:
 		return G_TYPE_STRING;
-	case FM_LIST_MODEL_ICON_COLUMN:
+	case FM_LIST_MODEL_SMALLEST_ICON_COLUMN:
+	case FM_LIST_MODEL_SMALLER_ICON_COLUMN:
+	case FM_LIST_MODEL_SMALL_ICON_COLUMN:
+	case FM_LIST_MODEL_STANDARD_ICON_COLUMN:
+	case FM_LIST_MODEL_LARGE_ICON_COLUMN:
+	case FM_LIST_MODEL_LARGER_ICON_COLUMN:
+	case FM_LIST_MODEL_LARGEST_ICON_COLUMN:
 		return GDK_TYPE_PIXBUF;
 	case FM_LIST_MODEL_FILE_NAME_IS_EDITABLE_COLUMN:
 		return G_TYPE_BOOLEAN;
@@ -158,7 +164,6 @@ fm_list_model_get_path (GtkTreeModel *tree_model, GtkTreeIter *iter)
 	return path;
 }
 
-
 static void
 fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column, GValue *value)
 {
@@ -166,7 +171,9 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	NautilusFile *file;
 	char *str;
 	GdkPixbuf *icon;
-	
+	int icon_size;
+	NautilusZoomLevel zoom_level;
+
 	model = (FMListModel *)tree_model;
 
 	g_return_if_fail (model->details->stamp == iter->stamp);
@@ -185,10 +192,18 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 		str = nautilus_file_get_string_attribute_with_default (file, "name");
 		g_value_set_string_take_ownership (value, str);
 		break;
-	case FM_LIST_MODEL_ICON_COLUMN:
+	case FM_LIST_MODEL_SMALLEST_ICON_COLUMN:
+	case FM_LIST_MODEL_SMALLER_ICON_COLUMN:
+	case FM_LIST_MODEL_SMALL_ICON_COLUMN:
+	case FM_LIST_MODEL_STANDARD_ICON_COLUMN:
+	case FM_LIST_MODEL_LARGE_ICON_COLUMN:
+	case FM_LIST_MODEL_LARGER_ICON_COLUMN:
+	case FM_LIST_MODEL_LARGEST_ICON_COLUMN:
 		g_value_init (value, GDK_TYPE_PIXBUF);
 
-		icon = nautilus_icon_factory_get_pixbuf_for_file (file, NULL, NAUTILUS_ICON_SIZE_SMALLER);
+		zoom_level = fm_list_model_get_zoom_level_from_column_id (column);
+		icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
+		icon = nautilus_icon_factory_get_pixbuf_for_file (file, NULL, icon_size);
 		g_value_set_object (value, icon);
 		g_object_unref (icon);
 		break;
@@ -339,30 +354,6 @@ typedef struct _SortTuple
   GSList *el;
 } SortTuple;
 
-static NautilusFileSortType
-fm_list_model_get_nautilus_file_sort_type (FMListModel *model)
-{
-	if (model->details->sort_column_id == -1) {
-		return NAUTILUS_FILE_SORT_NONE;
-	}
-
-	switch (model->details->sort_column_id) {
-	case FM_LIST_MODEL_NAME_COLUMN:
-		return NAUTILUS_FILE_SORT_BY_DISPLAY_NAME;
-	case FM_LIST_MODEL_SIZE_COLUMN:
-		return NAUTILUS_FILE_SORT_BY_SIZE;
-	case FM_LIST_MODEL_TYPE_COLUMN:
-		return NAUTILUS_FILE_SORT_BY_TYPE;
-	case FM_LIST_MODEL_DATE_MODIFIED_COLUMN:
-		return NAUTILUS_FILE_SORT_BY_MTIME;
-	default:
-		g_assert_not_reached ();
-	}
-
-	/* Should not be reached */
-	return NAUTILUS_FILE_SORT_NONE;
-}
-
 static int
 fm_list_model_compare_func (gconstpointer a,
 			    gconstpointer b,
@@ -379,7 +370,7 @@ fm_list_model_compare_func (gconstpointer a,
 	file2 = ((SortTuple *)b)->el->data;
 
 	result = nautilus_file_compare_for_sort (file1, file2,
-						 fm_list_model_get_nautilus_file_sort_type (model),
+						 fm_list_model_get_sort_type_from_sort_column_id (model->details->sort_column_id),
 						 model->details->sort_directories_first,
 						 (model->details->order == GTK_SORT_DESCENDING));
 
@@ -538,7 +529,7 @@ fm_list_model_add_file (FMListModel *model, NautilusFile *file)
 			file1 = tmp->data;
 
 			result = nautilus_file_compare_for_sort (file, file1,
-								 fm_list_model_get_nautilus_file_sort_type (model),
+								 fm_list_model_get_sort_type_from_sort_column_id (model->details->sort_column_id),
 								 model->details->sort_directories_first,
 								 (model->details->order == GTK_SORT_DESCENDING));
 			if (result < 0) {
@@ -680,6 +671,8 @@ fm_list_model_clear (FMListModel *model)
 {
 	GtkTreeIter iter;
 
+	g_return_if_fail (model != NULL);
+
 	while (model->details->files != NULL) {
 		iter.stamp = model->details->stamp;
 		iter.user_data = model->details->files;
@@ -753,6 +746,69 @@ fm_list_model_get_sort_column_id_from_sort_type (NautilusFileSortType sort_type)
 	g_return_val_if_reached (-1);
 }
 
+NautilusFileSortType
+fm_list_model_get_sort_type_from_sort_column_id (int sort_column_id)
+{
+	switch (sort_column_id) {
+	case FM_LIST_MODEL_NAME_COLUMN:
+		return NAUTILUS_FILE_SORT_BY_DISPLAY_NAME;
+	case FM_LIST_MODEL_TYPE_COLUMN:
+		return NAUTILUS_FILE_SORT_BY_TYPE;
+	case FM_LIST_MODEL_SIZE_COLUMN:
+		return NAUTILUS_FILE_SORT_BY_SIZE;
+	case FM_LIST_MODEL_DATE_MODIFIED_COLUMN:
+		return NAUTILUS_FILE_SORT_BY_MTIME;
+	}
+
+	g_return_val_if_reached (NAUTILUS_FILE_SORT_NONE);
+}
+
+NautilusZoomLevel
+fm_list_model_get_zoom_level_from_column_id (int column)
+{
+	switch (column) {
+	case FM_LIST_MODEL_SMALLEST_ICON_COLUMN:
+		return NAUTILUS_ZOOM_LEVEL_SMALLEST;
+	case FM_LIST_MODEL_SMALLER_ICON_COLUMN:
+		return NAUTILUS_ZOOM_LEVEL_SMALLER;
+	case FM_LIST_MODEL_SMALL_ICON_COLUMN:
+		return NAUTILUS_ZOOM_LEVEL_SMALL;
+	case FM_LIST_MODEL_STANDARD_ICON_COLUMN:
+		return NAUTILUS_ZOOM_LEVEL_STANDARD;
+	case FM_LIST_MODEL_LARGE_ICON_COLUMN:
+		return NAUTILUS_ZOOM_LEVEL_LARGE;
+	case FM_LIST_MODEL_LARGER_ICON_COLUMN:
+		return NAUTILUS_ZOOM_LEVEL_LARGER;
+	case FM_LIST_MODEL_LARGEST_ICON_COLUMN:
+		return NAUTILUS_ZOOM_LEVEL_LARGEST;
+	}
+
+	g_return_val_if_reached (NAUTILUS_ZOOM_LEVEL_STANDARD);
+}
+
+int
+fm_list_model_get_column_id_from_zoom_level (NautilusZoomLevel zoom_level)
+{
+	switch (zoom_level) {
+	case NAUTILUS_ZOOM_LEVEL_SMALLEST:
+		return FM_LIST_MODEL_SMALLEST_ICON_COLUMN;
+	case NAUTILUS_ZOOM_LEVEL_SMALLER:
+		return FM_LIST_MODEL_SMALLER_ICON_COLUMN;
+	case NAUTILUS_ZOOM_LEVEL_SMALL:
+		return FM_LIST_MODEL_SMALL_ICON_COLUMN;
+	case NAUTILUS_ZOOM_LEVEL_STANDARD:
+		return FM_LIST_MODEL_STANDARD_ICON_COLUMN;
+	case NAUTILUS_ZOOM_LEVEL_LARGE:
+		return FM_LIST_MODEL_LARGE_ICON_COLUMN;
+	case NAUTILUS_ZOOM_LEVEL_LARGER:
+		return FM_LIST_MODEL_LARGER_ICON_COLUMN;
+	case NAUTILUS_ZOOM_LEVEL_LARGEST:
+		return FM_LIST_MODEL_LARGEST_ICON_COLUMN;
+	}
+
+	g_return_val_if_reached (FM_LIST_MODEL_STANDARD_ICON_COLUMN);
+}
+
 static void
 fm_list_model_finalize (GObject *object)
 {
