@@ -31,6 +31,7 @@
 #include "fm-error-reporting.h"
 #include "fm-list-model.h"
 #include <eel/eel-cell-renderer-pixbuf-list.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtktreeselection.h>
@@ -70,6 +71,7 @@ static GtkTargetEntry drag_types [] = {
  */
 #define LIST_VIEW_MINIMUM_ROW_HEIGHT	28
 
+static int                      click_policy_auto_value;
 static NautilusFileSortType	default_sort_order_auto_value;
 static gboolean			default_sort_reversed_auto_value;
 static NautilusZoomLevel        default_zoom_level_auto_value;
@@ -78,6 +80,9 @@ static GList *              fm_list_view_get_selection         (FMDirectoryView
 static void                 fm_list_view_set_zoom_level        (FMListView *view,
 								NautilusZoomLevel new_level,
 								gboolean always_set_level);
+static void		    fm_list_view_scale_font_size       (FMListView *view, 
+								NautilusZoomLevel new_level,
+								gboolean update_size_table);
 
 GNOME_CLASS_BOILERPLATE (FMListView, fm_list_view,
 			 FMDirectoryView, FM_TYPE_DIRECTORY_VIEW)
@@ -92,21 +97,6 @@ list_selection_changed_callback (GtkTreeSelection *selection, gpointer user_data
 	fm_directory_view_notify_selection_changed (view);
 }
 
-static void
-list_activate_callback (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data)
-{
-	FMDirectoryView *view;
-	GList *file_list;
-	
-	view = FM_DIRECTORY_VIEW (user_data);
-
-	if (eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_DOUBLE) {
-		file_list = fm_list_view_get_selection (view);
-		fm_directory_view_activate_files (view, file_list);
-		nautilus_file_list_free (file_list);
-	}
-}
-
 /* Move these to eel? */
 
 static void
@@ -139,10 +129,18 @@ tree_view_has_selection (GtkTreeView *view)
 static void
 event_after_callback (GtkWidget *widget, GdkEventAny *event, gpointer callback_data)
 {
+	GList *file_list;
 	FMDirectoryView *view;
 
 	view = FM_DIRECTORY_VIEW (callback_data);
 
+	if (event->type == GDK_2BUTTON_PRESS &&
+	    click_policy_auto_value == NAUTILUS_CLICK_POLICY_DOUBLE) {
+		file_list = fm_list_view_get_selection (view);
+		fm_directory_view_activate_files (view, file_list);
+		nautilus_file_list_free (file_list);
+	}
+
 	if (event->type == GDK_BUTTON_PRESS
 	    && event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget))
 	    && (((GdkEventButton *) event)->button == 3)) {
@@ -198,7 +196,7 @@ button_release_callback (GtkWidget *widget, GdkEventButton *event, gpointer call
 
 	if (event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget)) &&
 	    event->button == 1 &&
-	    eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_SINGLE) {
+	    click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE) {
 		/* Handle single click activation preference. */
 		file_list = fm_list_view_get_selection (view);
 		fm_directory_view_activate_files (view, file_list);
@@ -208,25 +206,55 @@ button_release_callback (GtkWidget *widget, GdkEventButton *event, gpointer call
 	return FALSE;
 }
 
+static gboolean
+key_press_callback (GtkWidget *widget, GdkEventKey *event, gpointer callback_data)
+{
+	FMDirectoryView *view;
+	GList *file_list;
+
+	view = FM_DIRECTORY_VIEW (callback_data);
+
+	switch (event->keyval) {
+	case GDK_Return:
+	case GDK_KP_Enter:
+		file_list = fm_list_view_get_selection (view);
+		fm_directory_view_activate_files (view, file_list);
+		nautilus_file_list_free (file_list);
+		return TRUE;
+
+	default:
+		break;
+	}
+
+	return FALSE;
+}
+
 static void
-rows_reordered_callback (GtkTreeSortable *sortable, 
-			 FMListView *view)
+sort_column_changed_callback (GtkTreeSortable *sortable, 
+			      FMListView *view)
 {
 	NautilusFile *file;
 	gint sort_column_id;
-	GtkSortType order;
-	char *attr_column, *attr_order;
+	GtkSortType reversed;
+	char *sort_attr, *default_sort_attr;
+	char *reversed_attr, *default_reversed_attr;
 
 	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view));
-	gtk_tree_sortable_get_sort_column_id (sortable, &sort_column_id, &order);
 
-	attr_column = fm_list_model_get_attribute_from_sort_column_id (sort_column_id);
-	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_COLUMN, NULL, attr_column);
+	gtk_tree_sortable_get_sort_column_id (sortable, &sort_column_id, &reversed);
 
-	attr_order = (order ? "true" : "false");
-	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_REVERSED, NULL, attr_order);
+	sort_attr = fm_list_model_get_attribute_from_sort_column_id (sort_column_id);
+	sort_column_id = fm_list_model_get_sort_column_id_from_sort_type (default_sort_order_auto_value);
+	default_sort_attr = fm_list_model_get_attribute_from_sort_column_id (sort_column_id);
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_COLUMN,
+				    default_sort_attr, sort_attr);
+	g_free (default_sort_attr);
+	g_free (sort_attr);
 
-	g_free (attr_column);
+	default_reversed_attr = (default_sort_reversed_auto_value ? "true" : "false");
+	reversed_attr = (reversed ? "true" : "false");
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_REVERSED,
+				    default_reversed_attr, reversed_attr);
 }
 
 static void
@@ -256,24 +284,6 @@ cell_renderer_edited (GtkCellRendererText *cell,
 	nautilus_file_unref (file);
 }
 
-static void
-click_policy_changed_callback (gpointer callback_data)
-{
-	FMListView *view;
-
-	view = FM_LIST_VIEW (callback_data);
-
-	if (eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_SINGLE) {
-		g_object_set (G_OBJECT (view->details->file_name_cell),
-			      "underline", PANGO_UNDERLINE_SINGLE,
-			      NULL);
-	} else {
-		g_object_set (G_OBJECT (view->details->file_name_cell),
-			      "underline", PANGO_UNDERLINE_NONE,
-			      NULL);
-	}
-}
-
 static void
 create_and_set_up_tree_view (FMListView *view)
 {
@@ -286,23 +296,21 @@ create_and_set_up_tree_view (FMListView *view)
 				 "changed",
 				 G_CALLBACK (list_selection_changed_callback), view, 0);
 
-	g_signal_connect_object (view->details->tree_view, "row_activated",
-				 G_CALLBACK (list_activate_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "event-after",
 				 G_CALLBACK (event_after_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "button_press_event",
 				 G_CALLBACK (button_press_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "button_release_event",
 				 G_CALLBACK (button_release_callback), view, 0);
+	g_signal_connect_object (view->details->tree_view, "key_press_event",
+				 G_CALLBACK (key_press_callback), view, 0);
 	
 	view->details->model = g_object_new (FM_TYPE_LIST_MODEL, NULL);
 	gtk_tree_view_set_model (view->details->tree_view, GTK_TREE_MODEL (view->details->model));
 
 	g_signal_connect_object (view->details->model, "sort_column_changed",
-				 G_CALLBACK (rows_reordered_callback), view, 0);
+				 G_CALLBACK (sort_column_changed_callback), view, 0);
 
-	g_object_unref (view->details->model);
-	
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (view->details->tree_view), GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_rules_hint (view->details->tree_view, TRUE);
 
@@ -404,7 +412,16 @@ set_sort_order_from_metadata_and_preferences (FMListView *list_view)
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list_view->details->model),
 					      sort_column_id,
 					      sort_reversed ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
+}
 
+static gboolean
+list_view_changed_foreach (GtkTreeModel *model,
+              		   GtkTreePath  *path,
+			   GtkTreeIter  *iter,
+			   gpointer      data)
+{
+	gtk_tree_model_row_changed (model, path, iter);
+	return FALSE;
 }
 
 static void
@@ -419,6 +436,13 @@ set_zoom_level_from_metadata_and_preferences (FMListView *list_view)
 							    NAUTILUS_METADATA_KEY_LIST_VIEW_ZOOM_LEVEL, 
 							    default_zoom_level_auto_value);
 		fm_list_view_set_zoom_level (list_view, level, TRUE);
+		
+		/* reset the font size table for the new default zoom level */
+		fm_list_view_scale_font_size (list_view, level, TRUE);
+		
+		/* updated the rows after updating the font size */
+		gtk_tree_model_foreach (GTK_TREE_MODEL (list_view->details->model),
+					list_view_changed_foreach, NULL);
 	}
 }
 
@@ -443,7 +467,9 @@ fm_list_view_clear (FMDirectoryView *view)
 
 	list_view = FM_LIST_VIEW (view);
 	
-	fm_list_model_clear (list_view->details->model);
+	if (list_view->details->model != NULL) {
+		fm_list_model_clear (list_view->details->model);
+	}
 }
 
 static void
@@ -533,15 +559,124 @@ fm_list_view_reset_to_defaults (FMDirectoryView *view)
 	NautilusFile *file;
 
 	file = fm_directory_view_get_directory_as_file (view);
+
 	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_COLUMN, NULL, NULL);
 	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_REVERSED, NULL, NULL);
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_ZOOM_LEVEL, NULL, NULL);
 
 	gtk_tree_sortable_set_sort_column_id
 		(GTK_TREE_SORTABLE (FM_LIST_VIEW (view)->details->model),
 		 fm_list_model_get_sort_column_id_from_sort_type (default_sort_order_auto_value),
 		 default_sort_reversed_auto_value ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
 
-        fm_directory_view_restore_default_zoom_level (view);
+	fm_list_view_set_zoom_level (FM_LIST_VIEW (view), default_zoom_level_auto_value, FALSE);
+}
+
+static void
+fm_list_view_scale_font_size (FMListView *view, 
+					NautilusZoomLevel new_level,
+					gboolean update_size_table)
+{
+	static gboolean first_time = TRUE;
+	static double pango_scale[7];	
+
+	if (update_size_table || first_time) {
+		first_time = FALSE;
+	
+		switch (default_zoom_level_auto_value)
+		{
+		case NAUTILUS_ZOOM_LEVEL_LARGEST:
+			pango_scale[0] = (1 / 1.2) * (1 / 1.2) * PANGO_SCALE_XX_SMALL;
+			pango_scale[1] = (1 / 1.2) * PANGO_SCALE_XX_SMALL;
+			pango_scale[2] = PANGO_SCALE_XX_SMALL;
+			pango_scale[3] = PANGO_SCALE_X_SMALL;
+			pango_scale[4] = PANGO_SCALE_SMALL;
+			pango_scale[5] = PANGO_SCALE_MEDIUM;
+			pango_scale[6] = PANGO_SCALE_LARGE;
+			break;
+		case NAUTILUS_ZOOM_LEVEL_LARGER:
+			pango_scale[0] = (1 / 1.2) * PANGO_SCALE_XX_SMALL;
+			pango_scale[1] = PANGO_SCALE_XX_SMALL;
+			pango_scale[2] = PANGO_SCALE_X_SMALL;
+			pango_scale[3] = PANGO_SCALE_SMALL;
+			pango_scale[4] = PANGO_SCALE_MEDIUM;
+			pango_scale[5] = PANGO_SCALE_LARGE;
+			pango_scale[6] = PANGO_SCALE_X_LARGE;
+			break;
+		case NAUTILUS_ZOOM_LEVEL_LARGE:
+			pango_scale[0] = PANGO_SCALE_XX_SMALL;
+			pango_scale[1] = PANGO_SCALE_X_SMALL;
+			pango_scale[2] = PANGO_SCALE_SMALL;
+			pango_scale[3] = PANGO_SCALE_MEDIUM;
+			pango_scale[4] = PANGO_SCALE_LARGE;
+			pango_scale[5] = PANGO_SCALE_X_LARGE;
+			pango_scale[6] = PANGO_SCALE_XX_LARGE;
+			break;
+		case NAUTILUS_ZOOM_LEVEL_STANDARD:
+			pango_scale[0] = PANGO_SCALE_X_SMALL;
+			pango_scale[1] = PANGO_SCALE_SMALL;
+			pango_scale[2] = PANGO_SCALE_MEDIUM;
+			pango_scale[3] = PANGO_SCALE_LARGE;
+			pango_scale[4] = PANGO_SCALE_X_LARGE;
+			pango_scale[5] = PANGO_SCALE_XX_LARGE;
+			pango_scale[6] = 1.2 * PANGO_SCALE_XX_LARGE;
+			break;
+		case NAUTILUS_ZOOM_LEVEL_SMALL:
+			pango_scale[0] = PANGO_SCALE_SMALL;
+			pango_scale[1] = PANGO_SCALE_MEDIUM;
+			pango_scale[2] = PANGO_SCALE_LARGE;
+			pango_scale[3] = PANGO_SCALE_X_LARGE;
+			pango_scale[4] = PANGO_SCALE_XX_LARGE;
+			pango_scale[5] = 1.2 * PANGO_SCALE_XX_LARGE;
+			pango_scale[6] = 1.2 * 1.2 * PANGO_SCALE_XX_LARGE;
+			break;
+		case NAUTILUS_ZOOM_LEVEL_SMALLER:
+			/* From here on down we use PANGO_SCALE_MEDIUM for the
+			 * default zoom. Since the icons are now smaller this 
+			 * looks better at default zoom.
+			 */
+			pango_scale[0] = PANGO_SCALE_SMALL;
+			pango_scale[1] = PANGO_SCALE_MEDIUM;
+			pango_scale[2] = PANGO_SCALE_LARGE;
+			pango_scale[3] = PANGO_SCALE_X_LARGE;
+			pango_scale[4] = PANGO_SCALE_XX_LARGE;
+			pango_scale[5] = 1.2 * PANGO_SCALE_XX_LARGE;
+			pango_scale[6] = 1.2 * 1.2 * PANGO_SCALE_XX_LARGE;
+			break;
+		case NAUTILUS_ZOOM_LEVEL_SMALLEST:
+			pango_scale[0] = PANGO_SCALE_MEDIUM;
+			pango_scale[1] = PANGO_SCALE_LARGE;
+			pango_scale[2] = PANGO_SCALE_X_LARGE;
+			pango_scale[3] = PANGO_SCALE_XX_LARGE;
+			pango_scale[4] = 1.2 * PANGO_SCALE_XX_LARGE;
+			pango_scale[5] = 1.2 * 1.2 * PANGO_SCALE_XX_LARGE;
+			pango_scale[6] = 1.2 * 1.2 * 1.2 * PANGO_SCALE_XX_LARGE;
+			break;
+		default:
+			g_warning ("invalid default list-view zoom level");
+			pango_scale[0] = PANGO_SCALE_X_SMALL;
+			pango_scale[1] = PANGO_SCALE_SMALL;
+			pango_scale[2] = PANGO_SCALE_MEDIUM;
+			pango_scale[3] = PANGO_SCALE_LARGE;
+			pango_scale[4] = PANGO_SCALE_X_LARGE;
+			pango_scale[5] = PANGO_SCALE_XX_LARGE;
+			pango_scale[6] = 1.2 * PANGO_SCALE_XX_LARGE;
+			break;
+		}
+	}
+					 
+	g_object_set (G_OBJECT (view->details->file_name_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->size_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->type_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->date_modified_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
 }
 
 static void
@@ -549,13 +684,6 @@ fm_list_view_set_zoom_level (FMListView *view,
 			     NautilusZoomLevel new_level,
 			     gboolean always_set_level)
 {
-	static double pango_scale[7] = { PANGO_SCALE_X_SMALL,
-					 PANGO_SCALE_SMALL,
-					 PANGO_SCALE_MEDIUM,
-					 PANGO_SCALE_LARGE,
-					 PANGO_SCALE_X_LARGE,
-					 PANGO_SCALE_XX_LARGE,
-					 1.2 * PANGO_SCALE_XX_LARGE };
 	int icon_size;
 	int column;
 
@@ -587,18 +715,7 @@ fm_list_view_set_zoom_level (FMListView *view,
 					     NULL);
 
 	/* Scale text. */
-	g_object_set (G_OBJECT (view->details->file_name_cell),
-		      "scale", pango_scale[new_level],
-		      NULL);
-	g_object_set (G_OBJECT (view->details->size_cell),
-		      "scale", pango_scale[new_level],
-		      NULL);
-	g_object_set (G_OBJECT (view->details->type_cell),
-		      "scale", pango_scale[new_level],
-		      NULL);
-	g_object_set (G_OBJECT (view->details->date_modified_cell),
-		      "scale", pango_scale[new_level],
-		      NULL);
+	fm_list_view_scale_font_size (view, new_level, FALSE);
 
 	/* Make all rows the same size. */
 	icon_size = nautilus_get_icon_size_for_zoom_level (new_level);
@@ -686,6 +803,34 @@ fm_list_view_start_renaming_file (FMDirectoryView *view, NautilusFile *file)
 				  TRUE);
 }
 
+static void
+click_policy_changed_callback (gpointer callback_data)
+{
+	FMListView *view;
+
+	view = FM_LIST_VIEW (callback_data);
+
+	if (click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE) {
+		g_object_set (G_OBJECT (view->details->file_name_cell),
+			      "underline", PANGO_UNDERLINE_SINGLE,
+			      NULL);
+	} else {
+		g_object_set (G_OBJECT (view->details->file_name_cell),
+			      "underline", PANGO_UNDERLINE_NONE,
+			      NULL);
+	}
+}
+
+static void
+default_sort_order_changed_callback (gpointer callback_data)
+{
+	FMListView *list_view;
+
+	list_view = FM_LIST_VIEW (callback_data);
+
+	set_sort_order_from_metadata_and_preferences (list_view);
+}
+
 static void
 default_zoom_level_changed_callback (gpointer callback_data)
 {
@@ -707,6 +852,21 @@ fm_list_view_sort_directories_first_changed (FMDirectoryView *view)
 							 fm_directory_view_should_sort_directories_first (view));
 }
 
+static void
+fm_list_view_dispose (GObject *object)
+{
+	FMListView *list_view;
+
+	list_view = FM_LIST_VIEW (object);
+
+	if (list_view->details->model) {
+		g_object_unref (list_view->details->model);
+		list_view->details->model = NULL;
+	}
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static void
 fm_list_view_finalize (GObject *object)
 {
@@ -738,6 +898,7 @@ fm_list_view_class_init (FMListViewClass *class)
 
 	fm_directory_view_class = FM_DIRECTORY_VIEW_CLASS (class);
 
+	G_OBJECT_CLASS (class)->dispose = fm_list_view_dispose;
 	G_OBJECT_CLASS (class)->finalize = fm_list_view_finalize;
 
 	fm_directory_view_class->add_file = fm_list_view_add_file;
@@ -760,6 +921,8 @@ fm_list_view_class_init (FMListViewClass *class)
 	fm_directory_view_class->sort_directories_first_changed = fm_list_view_sort_directories_first_changed;
 	fm_directory_view_class->start_renaming_file = fm_list_view_start_renaming_file;
 
+	eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_CLICK_POLICY,
+				       &click_policy_auto_value);
 	eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_ORDER,
 				       (int *) &default_sort_order_auto_value);
 	eel_preferences_add_auto_boolean (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER,
@@ -775,14 +938,20 @@ fm_list_view_instance_init (FMListView *list_view)
 
 	create_and_set_up_tree_view (list_view);
 
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
-						  default_zoom_level_changed_callback,
-						  list_view, G_OBJECT (list_view));
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_CLICK_POLICY,
 						  click_policy_changed_callback,
 						  list_view, G_OBJECT (list_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_ORDER,
+						  default_sort_order_changed_callback,
+						  list_view, G_OBJECT (list_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER,
+						  default_sort_order_changed_callback,
+						  list_view, G_OBJECT (list_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
+						  default_zoom_level_changed_callback,
+						  list_view, G_OBJECT (list_view));
 
 	click_policy_changed_callback (list_view);
-
+	
 	fm_list_view_sort_directories_first_changed (FM_DIRECTORY_VIEW (list_view));
 }
