@@ -42,6 +42,7 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtkhbox.h>
+#include <gtk/gtkhbbox.h>
 #include <gtk/gtkhseparator.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
@@ -50,6 +51,7 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkvbox.h>
 #include <libgnome/gnome-i18n.h>
@@ -131,12 +133,14 @@ typedef struct {
 
 enum {
 	TARGET_URI_LIST,
-	TARGET_GNOME_URI_LIST
+	TARGET_GNOME_URI_LIST,
+	TARGET_RESET_BACKGROUND
 };
 
 static GtkTargetEntry target_table[] = {
 	{ "text/uri-list",  0, TARGET_URI_LIST },
-	{ "x-special/gnome-icon-list",  0, TARGET_GNOME_URI_LIST }
+	{ "x-special/gnome-icon-list",  0, TARGET_GNOME_URI_LIST },
+	{ "x-special/gnome-reset-background", 0, TARGET_RESET_BACKGROUND }
 };
 
 #define ERASE_EMBLEM_FILENAME	"erase.png"
@@ -269,6 +273,19 @@ uri_is_local_image (const char *uri)
 	return TRUE;
 }
 
+static void
+reset_icon (FMPropertiesWindow *properties_window)
+{
+	NautilusFile *file;
+	
+	file = properties_window->details->original_file;
+	
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_CUSTOM_ICON, NULL, NULL);
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_ICON_SCALE, NULL, NULL);
+	
+	gtk_widget_set_sensitive (properties_window->details->remove_image_button, FALSE);
+}
+
 static void  
 fm_properties_window_drag_data_received (GtkWidget *widget, GdkDragContext *context,
 					 int x, int y,
@@ -280,12 +297,18 @@ fm_properties_window_drag_data_received (GtkWidget *widget, GdkDragContext *cont
 	GtkImage *image;
  	GtkWindow *window; 
 
-	uris = g_strsplit (selection_data->data, "\r\n", 0);
-	exactly_one = uris[0] != NULL && uris[1] == NULL;
-
 	image = GTK_IMAGE (widget);
  	window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (image)));
 
+	if (info == TARGET_RESET_BACKGROUND) {
+		reset_icon (FM_PROPERTIES_WINDOW (window));
+		
+		return;
+	}
+	
+	uris = g_strsplit (selection_data->data, "\r\n", 0);
+	exactly_one = uris[0] != NULL && (uris[1] == NULL || uris[1][0] == '\0');
+
 
 	if (!exactly_one) {
 		eel_show_error_dialog
@@ -2135,6 +2158,9 @@ create_properties_window (StartupData *startup_data)
 {
 	FMPropertiesWindow *window;
 	GList *attributes;
+	GtkWidget *vbox;
+	GtkWidget *hbox;
+	GtkWidget *button;
 
 	window = FM_PROPERTIES_WINDOW (gtk_widget_new (fm_properties_window_get_type (), NULL));
 
@@ -2173,11 +2199,17 @@ create_properties_window (StartupData *startup_data)
 					 G_CALLBACK (properties_window_file_changed_callback),
 					 window, G_CONNECT_SWAPPED);
 
+	/* Create box for notebook and button box. */
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox);
+	gtk_container_add (GTK_CONTAINER (window), 
+			   GTK_WIDGET (vbox));
+
 	/* Create the notebook tabs. */
 	window->details->notebook = GTK_NOTEBOOK (gtk_notebook_new ());
 	gtk_widget_show (GTK_WIDGET (window->details->notebook));
-	gtk_container_add (GTK_CONTAINER (window), 
-			   GTK_WIDGET (window->details->notebook));
+	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (window->details->notebook),
+			    TRUE, TRUE, 0);
 
 	/* Create the pages. */
 	create_basic_page (window);
@@ -2189,6 +2221,23 @@ create_properties_window (StartupData *startup_data)
 	if (should_show_permissions (window)) {
 		create_permissions_page (window);
 	}
+	
+	/* Create box for close button. */
+	hbox = gtk_hbutton_box_new ();
+	gtk_widget_show (hbox);
+	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (hbox),
+			    FALSE, TRUE, 5);
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox),
+				   GTK_BUTTONBOX_END);  
+
+	/* Create close button. */
+	button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+ 	gtk_widget_show (button);
+	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (button),
+			    FALSE, TRUE, 0);
+	g_signal_connect_swapped (button, "clicked",
+				  G_CALLBACK (gtk_widget_destroy),
+				  window);
 
 	return window;
 }
@@ -2199,7 +2248,6 @@ get_target_file (NautilusFile *file)
 	NautilusFile *target_file;
 	char *uri;
 	char *uri_to_display;
-	char *local_path;
 
 	target_file = NULL;
 	if (nautilus_file_is_nautilus_link (file)) {
@@ -2207,15 +2255,15 @@ get_target_file (NautilusFile *file)
 		 * that seems fine since the links we care about are
 		 * all on the desktop.
 		 */
-		uri = nautilus_file_get_uri (file);
-		local_path = gnome_vfs_get_local_path_from_uri (uri);
-		if (local_path != NULL) {
-			switch (nautilus_link_local_get_link_type (local_path)) {
+		if (nautilus_file_is_local (file)) {
+			uri = nautilus_file_get_uri (file);
+
+			switch (nautilus_link_local_get_link_type (uri)) {
 			case NAUTILUS_LINK_MOUNT:
 			case NAUTILUS_LINK_TRASH:
 			case NAUTILUS_LINK_HOME:
 				/* map to linked URI for these types of links */
-				uri_to_display = nautilus_link_local_get_link_uri (local_path);
+				uri_to_display = nautilus_link_local_get_link_uri (uri);
 				target_file = nautilus_file_get (uri_to_display);
 				g_free (uri_to_display);
 				break;
@@ -2223,10 +2271,9 @@ get_target_file (NautilusFile *file)
 				/* don't for these types */
 				break;
 			}
-			g_free (local_path);
+			
+			g_free (uri);
 		}
-		
-		g_free (uri);
 	}
 
 	if (target_file != NULL) {
@@ -2432,7 +2479,8 @@ set_icon_callback (const char* icon_path, FMPropertiesWindow *properties_window)
 
 		/* re-enable the property window's clear image button */ 
 		gtk_widget_set_sensitive (properties_window->details->remove_image_button, TRUE);
-	}	
+	} else {
+	}
 }
 
 
